[![Crates.io](https://img.shields.io/crates/v/eui48.svg)](https://crates.io/crates/eui48)
[![docs.rs](https://docs.rs/eui48/badge.svg)](https://docs.rs/eui48)
{{badges}}

# {{crate}}

{{readme}}

## License
Licensed under the MIT/Apache-2.0 licenses. Please see the LICENSE-Apache and LICENSE-MIT files for more information.

### Contribution
Unless you explicitly state otherwise, any contribution intentionally submitted for inclusion in the work by you shall be licensed as above, without any additional terms or conditions.

